#include <memory>
#include <cstdio>
#include "SipHandler.hpp"
#include <sys/types.h>
#include <unistd.h>
#define TRACE(ARG) InfoLog ( << #ARG); ARG

using namespace resip;
using namespace std;


/**
 * Main Constructor for SipHandler
 *
 * @param host the ip address or hostname of the local machine
 * @param port port of host
 */
SipHandler::SipHandler (RtpManager* rtpm, char* host) : SipStack(false,0,true)
						     
{
  Data hostdata(host);
  mLocalUri.host() = hostdata;
  mLocalUri.port() = 5060;
  rtpman = rtpm;
  addTransport(UDP,5060,V4);
}

/**
 * SipHandler Thread function
 */
void SipHandler::thread () {
  while ( !isShutdown() ) {
    FdSet fdset;
    buildFdSet(fdset);
    int err = fdset.selectMilliSeconds(getTimeTillNextProcessMS());
    assert (err != -1);
    process(fdset);
    
    SipMessage* received = receive(); 
    if (received) { 
	processReceived(received);
	delete received;
    }
  }
}

/**
 * Verifies the SipMessage Request is an invite
 * @param msg Sip Request
 */
inline bool SipHandler::isInvite(SipMessage* msg) {
  return isType(msg,INVITE);
}

inline bool SipHandler::isType(SipMessage* msg, MethodTypes type) {
  assert(msg->isRequest());
  return (msg->header(h_RequestLine).getMethod()) == type;
}
/**
 * Processes received Sip Message
 * @param msg received sip message
 */
void SipHandler::processReceived(SipMessage* msg) {
  InfoLog (<< "stack1 got: " << msg->brief()
	   << " from user: " << msg->header(h_From).uri().user() 
	   << " at: " << msg->header(h_From).uri().host());
  if (msg->isResponse())
    handleResponse(msg);
  else if (msg->isRequest())
    handleRequest(msg);
  else
    unhandledMessage(msg);
}

/**
 * Handles a SipResponse
 * @param msg SipResponse
 */
void SipHandler::handleResponse(SipMessage* msg) {
  assert(msg->isResponse());
  unhandledMessage(msg);
}

/**
 * Handles a SipRequest
 * @param msg SipRequest
 */
void SipHandler::handleRequest(SipMessage* msg) {
  assert(msg->isRequest());
  if (isType(msg,INVITE))
    handleInvite(msg);
  else if (isType(msg,BYE))
    handleBye(msg);
  else if (isType(msg,OPTIONS))
    handleOptions(msg);
  else if (isType(msg,ACK))
    handleAck(msg);
  else
    unhandledMessage(msg);

}

/**
 * Handles an ACK SipResponse
 * @param msg SipResponse ACK
 */
void SipHandler::handleAck(SipMessage* msg) {
  SdpContents* sdp = (dynamic_cast<SdpContents*>(msg->getContents()));
  if (sdp)
    processSdp(msg,sdp);
}

void SipHandler::processSdp(SipMessage* msg, SdpContents* sdp) {
	std::string xmname(parseXm(msg));
	std::string identifier(msg->header(h_CallID).value().c_str());
	InfoLog(<< "Call ID in processSDP: " << identifier);
        std::cerr << *msg << std::endl;
  	int port = sdp->session().media().at(0).port();
  	std::string destip (sdp->session().connection().getAddress().c_str());  
  	InfoLog(<< "Destination IP: " << destip);
  	TRACE(rtpman->addDestination(xmname,identifier,destip,port));
}

void SipHandler::handleBye(SipMessage* msg) {
	std::string xmname(parseXm(msg));
	std::string identifier(msg->header(h_CallID).value().c_str());

	InfoLog(<< "Call ID in BYE: " << identifier);
	std::cerr << *msg << std::endl;
	TRACE(assert(rtpman->delDestination(xmname,identifier)));
	sendResponse(msg,200);
}

void SipHandler::handleOptions(SipMessage* msg) {
	sendResponse(msg,200);
}

void SipHandler::sendResponse(SipMessage* msg, int code, const Data& reason) {
  assert(msg->isRequest());
  SipMessage* response = resip::Helper::makeResponse(*msg, code,reason);
  send(*response);
  delete response;
}

/**
 * Handles a SIP INVITE
 * @param msg Sip INVITE
 */
void SipHandler::handleInvite(SipMessage* msg) {
  assert(isInvite(msg));
  NameAddr addr(mLocalUri);
  NameAddrs contact;
  int port = getPort(msg);

  
//   sendResponse(msg,200);

  SdpContents* sdp = (dynamic_cast<SdpContents*>(msg->getContents()));
  if (sdp)
    processSdp(msg,sdp);
  
  SipMessage* response = resip::Helper::makeResponse(*msg, 200);
  SdpContents contents = createSdpContents(port);

  response->setContents(&contents);
  contact.push_front(addr);
  
  response->header(h_Contacts) = contact;
  
  send(*response);
  delete response;
}

std::string SipHandler::parseXm(SipMessage* msg) {
	int location;
	Data user(msg->header(h_To).uri().user());
	InfoLog(<< "Full user: " << user);
	// Ok time to strip the xmXXX or iXXXX out of the To Header
	if ((location = user.find("x",1)) == Data::npos)
		location = user.find("i",1);
	std::string xm(user.substr(location).c_str());
	InfoLog(<< "Substring result: " << xm);
	return xm;
}
	
int SipHandler::getPort(SipMessage* msg) {
	std::string xmname(parseXm(msg));
	return rtpman->getPort(xmname);
}

/**
 * Creates the SDP part of the Sip Message
 * @return SDP Contents 
 */
SdpContents SipHandler::createSdpContents (int port) {
  SdpContents sdp;
  sdp.session() = createSdpContentsSession(port);
  return sdp;
}

/**
 * Creates Session information for SDP
 * @return Session Information
 */
SdpContents::Session SipHandler::createSdpContentsSession (int port) {
  SdpContents::Session session;  
  Data ip(mLocalUri.host());
  
  resip::SdpContents::Session::Origin origin(Data("moh"),getpid(),getpid(),(resip::SdpContents::AddrType)1,ip);
  resip::SdpContents::Session::Connection con((resip::SdpContents::AddrType)1,ip);

  session.addMedium(createMedium(port));
  session.name() = "session";
  session.connection() = con;
  session.origin() = origin;
  
  return session;
}

/**
 * Creates Media Session information for SDP
 * @return Media Session
 */
SdpContents::Session::Medium SipHandler::createMedium (int port) {
  Data name("audio");
  Data protocol("RTP/AVP");
  SdpContents::Session::Medium med(name,port,0,protocol);
  
  med.addCodec(resip::SdpContents::Session::Codec::ULaw_8000);
  return med;
}

void SipHandler::unhandledMessage(SipMessage* msg) {
  DebugLog (<< "UNHANDLED" << *msg);
}

SipHandler::~SipHandler () {}

