#ifndef SipHandler_HPP
#define SipHandler_HPP
#include "resiprocate/SipStack.hxx"
#include "resiprocate/os/Data.hxx"
#include "resiprocate/os/ThreadIf.hxx"
#include "resiprocate/Helper.hxx"
#include "resiprocate/SipMessage.hxx"
#include "resiprocate/SipStack.hxx"
#include "resiprocate/SdpContents.hxx"
#include "resiprocate/Uri.hxx"
#include "resiprocate/Connection.hxx"
#include "resiprocate/os/Logger.hxx"
#include "resiprocate/os/Tuple.hxx"
#include "RtpWrapper.hpp"
#include "RtpManager.hpp"
#include <string>

/**
 * @file   SipHandler.hpp
 * @author Dan Weber
 * @date   Sun Mar  6 21:43:26 2005
 * 
 * @brief  Sip Handling Class
 * 
 * 
 */


#define RESIPROCATE_SUBSYSTEM Subsystem::TEST
/// Sip Handling Class
class SipHandler : public resip::SipStack, public resip::ThreadIf  {
public:
  /** 
   * Constructs a SipHandler instance for the application
   * 
   * @param rtpm Rtp Management Instance
   * @param host Local Host IP
   * 
   * @todo This should be a Singleton
   */
  SipHandler (RtpManager* rtpm, char* host);
  /// Default Destructor
  virtual ~SipHandler ();
  /// Thread Function
  void thread();
private:
  /** 
   * Wrapper around isType for an INVITE
   * 
   * @param msg Sip Message to compare
   * 
   * @return True if it is an INVITE
   */
  bool isInvite(resip::SipMessage* msg);
  /** 
   * Compares the message <B>request</B> to the given type
   * 
   * @param msg Sip Message Request
   * @param type Type of Message Request
   * 
   * @return Ture if its equivalent
   */
  bool isType(resip::SipMessage* msg, resip::MethodTypes type);
  /** 
   * Calls respective Request or Response functions depending upon the type
   * 
   * @param msg Sip Message
   */
  void processReceived(resip::SipMessage* msg);
  /** 
   * Handles a Sip Message Request
   * 
   * @param msg Sip Message Request
   */
  void handleRequest(resip::SipMessage* msg);
  /** 
   * Handle a Sip Message Response
   * 
   * @param msg Sip Message Response
   */
  void handleResponse(resip::SipMessage* msg);
  /** 
   * Handles a SIP Invite
   * 
   * @param msg SIP Invite Message
   */
  void handleInvite(resip::SipMessage* msg);
  /** 
   * Handles an ACK Message
   * 
   * @param msg SIP ACK Message
   */
  void handleAck(resip::SipMessage* msg);
  /** 
   * Handles a BYE Request
   * 
   * @param msg SIP BYE Message
   */
  void handleBye(resip::SipMessage* msg);
  /** 
   * Handles an OPTIONS Request
   * 
   * @param msg SIP OPTIONS Message
   */
  void handleOptions(resip::SipMessage* msg);
  /** 
   * Parses the XM Name out of the Sip Message
   * 
   * @param msg Sip Message
   * 
   * @return XM Name
   */
  std::string parseXm(resip::SipMessage* msg);
  /** 
   * Processes the SDP from a Sip INVITE or ACK
   * 
   * @param msg Sip Message
   * @param sdp SDP
   */
  void processSdp(resip::SipMessage* msg, resip::SdpContents* sdp);
  /** 
   * Retrieves the port from the given Sip Message
   * 
   * @param msg Sip Message
   * 
   * @return Port number
   */
  int getPort(resip::SipMessage* msg);
  /** 
   * Sends a resposne for the given message
   * 
   * @param msg Sip Message
   * @param code Response code
   * @param reason Reason for Response
   */
  void sendResponse(resip::SipMessage* msg, int code, const resip::Data& reason=resip::Data::Empty);
  resip::SdpContents createSdpContents(int);
  resip::SdpContents::Session createSdpContentsSession(int);
  resip::SdpContents::Session::Medium createMedium(int);
  void unhandledMessage(resip::SipMessage* msg);
private:
  /// Local URI
  resip::Uri mLocalUri;
  /// RtpManager Instance
  RtpManager* rtpman;
};
#endif
